@@ -79,7 +79,18 @@ const wchar_t AppUi::className[] = L"ipeWindowClass";
 static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
 {
   String dir = ipeIconDirectory();
-  String fname = dir + action + ".png";
+  String base = dir + action;
+  String fname;
+  int ww = w < 0 ? -w : w;
+  if (ww >= 64 && Platform::fileExists(base + "_32x32@2x.png"))
+    fname = base + "_32x32@2x.png";
+  else if (ww >= 48 && Platform::fileExists(base + "@2x.png"))
+    fname = base + "@2x.png";
+  else if (ww >= 32 && Platform::fileExists(base + "_32x32.png"))
+    fname = base + "_32x32.png";
+  else
+    fname = base + ".png";
+  // ipeDebug("loadIcon %s at %d x %d from %s", action.z(), w, h, fname.z());
   if (!Platform::fileExists(fname))
     return NULL;
 
@@ -156,9 +167,10 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
   return bm;
 }
 
-static int loadIcon(String action, HIMAGELIST il)
+static int loadIcon(String action, HIMAGELIST il, int scale)
 {
-  HBITMAP bm = loadIcon(action, TBICONSIZE, TBICONSIZE, -1, -1, -1);
+  int size = scale * TBICONSIZE / 100;
+  HBITMAP bm = loadIcon(action, size, size, -1, -1, -1);
   if (bm == NULL)
     return -1;
   int r = ImageList_Add(il, bm, NULL);
@@ -166,23 +178,24 @@ static int loadIcon(String action, HIMAGELIST il)
   return r;
 }
 
-static HBITMAP loadButtonIcon(String action)
+static HBITMAP loadButtonIcon(String action, int scale)
 {
+  int size = scale * 24 / 100;
   DWORD rgb = GetSysColor(COLOR_BTNFACE);
   int r0 = GetRValue(rgb);
   int g0 = GetGValue(rgb);
   int b0 = GetBValue(rgb);
-  return loadIcon(action, -1, -1, r0, g0, b0);
+  return loadIcon(action, -size, -1, r0, g0, b0);
 }
 
-static HBITMAP colorIcon(Color color)
+static HBITMAP colorIcon(Color color, int scale)
 {
   int r = int(color.iRed.internal() * 255 / 1000);
   int g = int(color.iGreen.internal() * 255 / 1000);
   int b = int(color.iBlue.internal() * 255 / 1000);
   COLORREF rgb = RGB(r, g, b);
-  int cx = 22;
-  int cy = 22;
+  int cx = scale * 22 / 100;
+  int cy = scale * 22 / 100;
   HDC hdc = GetDC(NULL);
   HDC memDC = CreateCompatibleDC(hdc);
   HBITMAP bm = CreateCompatibleBitmap(hdc, cx, cy);
@@ -211,7 +224,7 @@ void AppUi::createAction(String name, String tooltip, bool canWhileDrawing)
 {
   SAction s;
   s.name = String(name);
-  s.icon = loadIcon(name, hIcons);
+  s.icon = loadIcon(name, hIcons, iToolbarScale);
   s.tooltip = tooltip;
   s.alwaysOn = canWhileDrawing;
   iActions.push_back(s);
@@ -397,11 +410,27 @@ AppUi::AppUi(lua_State *L0, int model)
   iFullScreen = false;
 }
 
+static void insert_tb(HWND hRebar, REBARBANDINFO &rbBand, HWND tb, int size)
+{
+  rbBand.hwndChild = tb;
+  rbBand.cxMinChild = size;
+  rbBand.cx = size;
+  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
+}
+
 // called during Window creation
 void AppUi::initUi()
 {
-  hIcons = ImageList_Create(TBICONSIZE, TBICONSIZE, ILC_COLOR32, 20, 4);
-  hColorIcons = ImageList_Create(22, 22, ILC_COLOR32, 20, 4);
+  // update scale if dpi information is available
+  int dpi = Canvas::getDpiForWindow(hwnd);
+  if (iUiScale == 100)
+    iUiScale = dpi;
+  if (iToolbarScale == 100)
+    iToolbarScale = dpi;
+
+  int tbsize = iToolbarScale * TBICONSIZE / 100;
+  hIcons = ImageList_Create(tbsize, tbsize, ILC_COLOR32, 20, 4);
+  hColorIcons = ImageList_Create(uiscale(22), uiscale(22), ILC_COLOR32, 20, 4);
   HINSTANCE hInst = (HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE);
 
   hMenuBar = CreateMenu();
@@ -434,29 +463,42 @@ void AppUi::initUi()
   addTButton(hSnapTools, "snapangle", BTNS_CHECK);
   addTButton(hSnapTools, "snapauto", BTNS_CHECK);
 
-#define SNAP_BUTTONS 6
+#define SNAP_BUTTONS 7
 
   hEditTools = createToolBar(hInst);
+  int EDIT_BUTTONS = 0;
 
-  addTButton(hEditTools, "copy");
-  addTButton(hEditTools, "cut");
-  addTButton(hEditTools, "paste");
-  addTButton(hEditTools, "delete");
+  if (iLeftDockFloats) {
+    createAction("dock_visible", "Show properties and layers", true);
+    addTButton(hEditTools, "dock_visible", BTNS_CHECK);
+    EDIT_BUTTONS += 1;
+  }
+
+  if (!isMiniEdit) {
+    addTButton(hEditTools, "copy");
+    addTButton(hEditTools, "cut");
+    addTButton(hEditTools, "paste");
+    addTButton(hEditTools, "delete");
+    EDIT_BUTTONS += 4;
+  }
   addTButton(hEditTools, "undo");
   addTButton(hEditTools, "redo");
-  addTButton(hEditTools, "zoom_in");
-  addTButton(hEditTools, "zoom_out");
-  addTButton(hEditTools, "fit_objects");
-  addTButton(hEditTools, "fit_page");
-  addTButton(hEditTools, "fit_width");
-  addTButton(hEditTools, "keyboard");
+  EDIT_BUTTONS += 2;
+  if (!isMiniEdit) {
+    addTButton(hEditTools, "zoom_in");
+    addTButton(hEditTools, "zoom_out");
+    addTButton(hEditTools, "fit_objects");
+    addTButton(hEditTools, "fit_page");
+    addTButton(hEditTools, "fit_width");
+    addTButton(hEditTools, "keyboard");
+    EDIT_BUTTONS += 6;
+  }
   createAction("shift_key", "Press the Shift key", true);
   addTButton(hEditTools, "shift_key", BTNS_CHECK);
   addTButton(hEditTools, "grid_visible", BTNS_CHECK);
   createAction("stop", "Abort object being drawn [Esc]", true);
   addTButton(hEditTools, "stop");
-
-#define EDIT_BUTTONS 15
+  EDIT_BUTTONS += 3;
 
   hObjectTools = createToolBar(hInst);
 
@@ -464,6 +506,8 @@ void AppUi::initUi()
   addTButton(hObjectTools, "mode_translate", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_rotate", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_stretch", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_shear", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_graph", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_pan", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_shredder", BTNS_CHECKGROUP);
   addTButton(hObjectTools, 0, BTNS_CHECKGROUP);
@@ -471,7 +515,10 @@ void AppUi::initUi()
   addTButton(hObjectTools, "mode_math", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_paragraph", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_marks", BTNS_CHECKGROUP);
-  addTButton(hObjectTools, "mode_rectangles", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_rectangles1", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_rectangles2", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_rectangles3", BTNS_CHECKGROUP);
+  addTButton(hObjectTools, "mode_parallelogram", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_lines", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_polygons", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_splines", BTNS_CHECKGROUP);
@@ -484,14 +531,14 @@ void AppUi::initUi()
   addTButton(hObjectTools, "mode_circle3", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_ink", BTNS_CHECKGROUP);
 
-#define MODE_BUTTONS 22
+#define MODE_BUTTONS 24
 
   // size is used for dropdown list!  Don't set to zero.
   for (int i = EUiGridSize; i <= EUiAngleSize; ++i)
     hSelector[i] =
       CreateWindowExW(0, WC_COMBOBOXEXW, NULL,
 		      WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST,
-		      0, 0, 100, 300, hwnd,
+		      0, 0, uiscale(100), uiscale(300), hwnd,
 		      (HMENU) (ID_SELECTOR_BASE + i), hInst, NULL);
 
   hRebar = CreateWindowEx(WS_EX_TOOLWINDOW, REBARCLASSNAME, NULL,
@@ -516,31 +563,31 @@ void AppUi::initUi()
   rbBand.fStyle = RBBS_CHILDEDGE | RBBS_GRIPPERALWAYS | RBBS_HIDETITLE;
   rbBand.cyMinChild = HIWORD(dwBtnSize) + 4;
   rbBand.cyChild = HIWORD(dwBtnSize) + 4;
+  int bw = LOWORD(dwBtnSize);
 
-  rbBand.hwndChild = hSnapTools;
-  rbBand.cxMinChild = SNAP_BUTTONS * LOWORD(dwBtnSize);
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hSelector[EUiGridSize];
-  rbBand.cxMinChild = 160;
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hSelector[EUiAngleSize];
-  rbBand.cxMinChild = 100;
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hEditTools;
-  rbBand.cxMinChild = EDIT_BUTTONS * LOWORD(dwBtnSize);
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
-
-  rbBand.hwndChild = hObjectTools;
-  rbBand.cxMinChild = MODE_BUTTONS * LOWORD(dwBtnSize);
-  rbBand.cx = rbBand.cxMinChild;
-  SendMessage(hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM) &rbBand);
+  lua_getglobal(L, "prefs");
+  lua_getfield(L, -1, "toolbar_order");
+  if (lua_istable(L, -1)) {
+    int n = lua_rawlen(L, -1);
+    for (int i = 1; i <= n; ++i) {
+      lua_rawgeti(L, -1, i);
+      if (lua_isstring(L, -1)) {
+	String s = lua_tostring(L, -1);
+	if (s == "edit")
+	  insert_tb(hRebar, rbBand, hEditTools, EDIT_BUTTONS * bw);
+	else if (s == "grid")
+	  insert_tb(hRebar, rbBand, hSelector[EUiGridSize], 160);
+	else if (s == "angle")
+	  insert_tb(hRebar, rbBand, hSelector[EUiAngleSize], 100);
+	else if (s == "snap")
+	  insert_tb(hRebar, rbBand, hSnapTools, SNAP_BUTTONS * bw);
+	else if (s == "mode")
+	  insert_tb(hRebar, rbBand, hObjectTools, MODE_BUTTONS * bw);
+      }
+      lua_pop(L, 1);
+    }
+  }
+  lua_pop(L, 2); // toolbar_order, prefs
 
   // ------------------------------------------------------------
 
@@ -556,25 +603,25 @@ void AppUi::initUi()
   for (int i = 0; i < EUiGridSize; ++i)
     hSelector[i] = CreateWindowExW(0, WC_COMBOBOXEXW, NULL,
 				   WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST,
-				   0, 0, 100, 300, hwnd,
+				   0, 0, uiscale(100), uiscale(300), hwnd,
 				   (HMENU) (ID_SELECTOR_BASE + i), hInst, NULL);
   SendMessage(hButton[EUiPen], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("pen"));
+	      (LPARAM) loadButtonIcon("pen", iUiScale));
   SendMessage(hButton[EUiTextSize], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("mode_label"));
+	      (LPARAM) loadButtonIcon("mode_label", iUiScale));
   SendMessage(hButton[EUiSymbolSize], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("mode_marks"));
+	      (LPARAM) loadButtonIcon("mode_marks", iUiScale));
   SendMessage(hButton[EUiStroke], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) colorIcon(Color(1000, 0, 0)));
+	      (LPARAM) colorIcon(Color(1000, 0, 0), iUiScale));
   SendMessage(hButton[EUiFill], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) colorIcon(Color(1000, 1000, 0)));
+	      (LPARAM) colorIcon(Color(1000, 1000, 0), iUiScale));
 
   hButton[EUiMarkShape] =
     CreateWindowExW(0, L"static", NULL,
 		    WS_CHILD | WS_VISIBLE | SS_BITMAP | SS_CENTERIMAGE,
 		    0, 0, 0, 0, hwnd, NULL, hInst, NULL);
   SendMessage(hButton[EUiMarkShape], STM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) loadButtonIcon("mode_marks"));
+	      (LPARAM) loadButtonIcon("mode_marks", iUiScale));
 
   iPathView = new PathView(hwnd, ID_PATHVIEW);
 
@@ -649,11 +696,12 @@ void AppUi::initUi()
   setTooltip(hLayers, "Layers of this page");
 
   setCheckMark("coordinates|", "points");
+  setCheckMark("scaling|", "1");
   setCheckMark("mode_", "select");
 
   SetFocus(hwndCanvas);
 
-  hFont = CreateFontW(18, 0, 0, 0, FW_DONTCARE,
+  hFont = CreateFontW(uiscale(18), 0, 0, 0, FW_DONTCARE,
 		      FALSE, FALSE, FALSE, ANSI_CHARSET,
 		      OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
 		      DEFAULT_QUALITY,
@@ -672,6 +720,11 @@ void AppUi::initUi()
   // default is on, Lua code will turn off if necessary
   CheckMenuItem(hMenuBar, actionId("toggle_notes"), MF_CHECKED);
   CheckMenuItem(hMenuBar, actionId("toggle_bookmarks"), MF_CHECKED);
+
+  if (iLeftDockFloats)
+    // SendMessage(hEditTools, TB_SETSTATE, actionId("dock_visible"),
+    // (LPARAM) (TBSTATE_ENABLED | TBSTATE_CHECKED));
+    setLeftDockVisibility(false);
 }
 
 AppUi::~AppUi()
@@ -695,58 +748,82 @@ void AppUi::layoutChildren(bool resizeRebar)
   int sbh = rc1.bottom - rc1.top;
   int tbh = rc2.bottom - rc2.top;
   int cvh = rc.bottom - sbh - tbh;
+
   RECT rect;
-  rect.left = 0;
-  rect.top = tbh;
-  rect.right = 200;
-  rect.bottom = rc.bottom - sbh;
-  InvalidateRect(hwnd, &rect, TRUE);
-  int pbh = 28 * 7 + 70;
-  MoveWindow(hProperties, 8, tbh + 2, 180, pbh, TRUE);
+  bool visLeftDock = IsWindowVisible(hProperties);
+  if (visLeftDock) {
+    rect.left = 0;
+    rect.top = tbh;
+    rect.right = uiscale(200);
+    rect.bottom = rc.bottom - sbh;
+    InvalidateRect(hwnd, &rect, TRUE);
 
-  for (int i = 0; i <= EUiSymbolSize; ++i) {
-    int y = 24 + 28 * i + (i > 2 ? 42 : 0);
-    MoveWindow(hButton[i], 16, tbh + y, 26, 26, TRUE);
-    MoveWindow(hSelector[i], 50, tbh + y, 132, 26, TRUE);
+    int dy = 28 + iUiGap;
+    int pbh = uiscale(dy * 6 + 28 + 70);
+    MoveWindow(hProperties, uiscale(8), tbh + uiscale(2),
+	       uiscale(180), pbh, TRUE);
+    for (int i = 0; i <= EUiSymbolSize; ++i) {
+      int y = uiscale(24 + dy * i + (i > 2 ? 42 : 0));
+      MoveWindow(hButton[i], uiscale(16), tbh + y,
+		 uiscale(26), uiscale(26), TRUE);
+      MoveWindow(hSelector[i], uiscale(50), tbh + y,
+		 uiscale(132), uiscale(26), TRUE);
+    }
+
+    int y = tbh + uiscale(24 + dy * 6 + 42);
+    MoveWindow(hViewNumber, uiscale(16), y, uiscale(68), uiscale(26), TRUE);
+    MoveWindow(hPageNumber, uiscale(116), y, uiscale(68), uiscale(26), TRUE);
+    MoveWindow(hViewMarked, uiscale(85), y + uiscale(5),
+	       uiscale(12), uiscale(12), TRUE);
+    MoveWindow(hPageMarked, uiscale(100), y + uiscale(5),
+	       uiscale(12), uiscale(12), TRUE);
+
+    MoveWindow(iPathView->windowId(), uiscale(16),
+	       tbh + uiscale(24 + dy * 2 + 28 + iUiGap / 2),
+	       uiscale(168), uiscale(40), TRUE);
+
+    MoveWindow(hLayerGroup, uiscale(8), tbh + uiscale(6) + pbh,
+	       uiscale(180), cvh - pbh - uiscale(8), TRUE);
+    MoveWindow(hLayers, uiscale(16), tbh + uiscale(24) + pbh,
+	       uiscale(164), cvh - pbh - uiscale(32), TRUE);
   }
 
-  int y = tbh + 24 + 28 * 6 + 42;
-  MoveWindow(hViewNumber, 16, y, 68, 26, TRUE);
-  MoveWindow(hPageNumber, 116, y, 68, 26, TRUE);
-  MoveWindow(hViewMarked, 85, y + 5, 12, 12, TRUE);
-  MoveWindow(hPageMarked, 100, y + 5, 12, 12, TRUE);
-
-  MoveWindow(iPathView->windowId(), 16, tbh + 24 + 28 * 3, 168, 40, TRUE);
-
-  MoveWindow(hLayerGroup, 8, tbh + 6 + pbh, 180, cvh - pbh - 8, TRUE);
-  MoveWindow(hLayers, 16, tbh + 24 + pbh, 164, cvh - pbh - 32, TRUE);
-
   bool visNotes = IsWindowVisible(hNotes);
   bool visBookmarks = IsWindowVisible(hBookmarks);
-  int cvw = (visNotes || visBookmarks) ? rc.right - 400 : rc.right - 200;
-  MoveWindow(hwndCanvas, 200, tbh, cvw, cvh, TRUE);
+  int cvl = visLeftDock ? uiscale(200) : 0;
+  int cvw = rc.right - cvl -
+    ((visNotes || visBookmarks) ? uiscale(iWidthNotesBookmarks) : 0);
+  MoveWindow(hwndCanvas, cvl, tbh, cvw, cvh, TRUE);
   if (visNotes || visBookmarks) {
-    rect.left = rc.right - 200;
+    rect.left = rc.right - uiscale(iWidthNotesBookmarks);
     rect.top = tbh;
     rect.right = rc.right;
     rect.bottom = rc.bottom - sbh;
     InvalidateRect(hwnd, &rect, TRUE);
   }
 
-  int nth = (visNotes && visBookmarks) ? cvh / 2 - 4: cvh;
+  int nth = (visNotes && visBookmarks) ? cvh / 2 - uiscale(4): cvh;
   int nty = tbh;
   if (visNotes) {
-    MoveWindow(hNotesGroup, rc.right - 200, nty, 200, nth, TRUE);
-    MoveWindow(hNotes, rc.right - 194, nty + 18, 186, nth - 24, TRUE);
-    nty += nth + 2;
+    MoveWindow(hNotesGroup, rc.right - uiscale(iWidthNotesBookmarks),
+	       nty, uiscale(iWidthNotesBookmarks), nth, TRUE);
+    MoveWindow(hNotes, rc.right - uiscale(iWidthNotesBookmarks - 6),
+	       nty + uiscale(18), uiscale(iWidthNotesBookmarks - 14),
+	       nth - uiscale(24), TRUE);
+    nty += nth + uiscale(2);
   }
   if (visBookmarks) {
-    MoveWindow(hBookmarksGroup, rc.right - 200, nty, 200, nth, TRUE);
-    MoveWindow(hBookmarks, rc.right - 194, nty + 18, 186, nth - 24, TRUE);
+    MoveWindow(hBookmarksGroup, rc.right - uiscale(iWidthNotesBookmarks),
+	       nty, uiscale(iWidthNotesBookmarks), nth, TRUE);
+    MoveWindow(hBookmarks, rc.right - uiscale(iWidthNotesBookmarks - 6),
+	       nty + uiscale(18),
+	       uiscale(iWidthNotesBookmarks - 14), nth - uiscale(24), TRUE);
   }
-  int statwidths[] = {rc.right - 300, rc.right - 200, rc.right - 80, rc.right };
+  int statwidths[] = {rc.right - uiscale(320), rc.right - uiscale(220),
+		      rc.right - uiscale(80), rc.right };
   SendMessage(hStatusBar, SB_SETPARTS, 4, (LPARAM) statwidths);
   iCanvas->update();
+  BringWindowToTop(hProperties);
 }
 
 // --------------------------------------------------------------------
@@ -777,7 +854,7 @@ void AppUi::addComboColors(AttributeSeq &sym, AttributeSeq &abs)
   item.iItem = (INT_PTR) -1;
   for (uint i = 0; i < sym.size(); ++i) {
     Color color = abs[i].color();
-    HBITMAP bm = colorIcon(color);
+    HBITMAP bm = colorIcon(color, iUiScale);
     ImageList_Add(hColorIcons, bm, NULL);
     DeleteObject(bm);
     String s = sym[i].string();
@@ -808,7 +885,7 @@ void AppUi::setButtonColor(int sel, Color color)
 {
   // ipeDebug("Changing color for %d");
   SendMessage(hButton[sel], BM_SETIMAGE, IMAGE_BITMAP,
-	      (LPARAM) colorIcon(color));
+	      (LPARAM) colorIcon(color, iUiScale));
 }
 
 void AppUi::setPathView(const AllAttributes &all, Cascade *sheet)
@@ -1149,6 +1226,28 @@ void AppUi::aboutIpe()
 	      MB_OK | MB_ICONINFORMATION | MB_APPLMODAL);
 }
 
+void AppUi::setLeftDockVisibility(bool vis)
+{
+  int cmd = vis ?SW_SHOW : SW_HIDE;
+  ShowWindow(hProperties, cmd);
+  ShowWindow(hLayerGroup, cmd);
+  ShowWindow(hLayers, cmd);
+  for (int i = 0; i < EUiGridSize; ++i)
+    ShowWindow(hButton[i], cmd);
+  for (int i = 0; i < EUiView; ++i)
+    if (i != EUiGridSize && i != EUiAngleSize)
+      ShowWindow(hSelector[i], cmd);
+  ShowWindow(hViewNumber, cmd);
+  ShowWindow(hPageNumber, cmd);
+  ShowWindow(hViewMarked, cmd);
+  ShowWindow(hPageMarked, cmd);
+  ShowWindow(iPathView->windowId(), cmd);
+  layoutChildren(false);
+  auto pan = iCanvas->pan();
+  double delta = (vis ? +1 : -1) * uiscale(100) / iCanvas->zoom();
+  iCanvas->setPan(pan + Vector(delta, 0.0));
+}
+
 void AppUi::action(String name)
 {
   ipeDebug("action %s", name.z());
@@ -1165,6 +1264,10 @@ void AppUi::action(String name)
 	mod |= CanvasBase::EShift;
       iCanvas->setAdditionalModifiers(mod);
     }
+  } else if (name == "dock_visible") {
+    bool vis = (SendMessage(hEditTools, TB_GETSTATE, (WPARAM) id, 0)
+		& TBSTATE_CHECKED);
+    setLeftDockVisibility(vis);
   } else {
     // Implement radio buttons
     int i = name.find("|");
@@ -1312,8 +1415,7 @@ int AppUi::clipboard(lua_State *L)
       CloseClipboard();
       return 0;
     }
-    wchar_t *q = (wchar_t *) GlobalLock(hGlobal);
-    String s = Platform::wideToUtf8(q);
+    String s((wchar_t *) GlobalLock(hGlobal));
     GlobalUnlock(hGlobal);
     CloseClipboard();
     lua_pushstring(L, s.z());
